@@ -12,7 +12,7 @@ from glob import has_magic
 from .callbacks import _DEFAULT_CALLBACK
 from .exceptions import FSTimeoutError
 from .spec import AbstractFileSystem
-from .utils import PY36, is_exception, other_paths
+from .utils import is_exception, other_paths
 
 private = re.compile("_[^_]")
 
@@ -29,12 +29,6 @@ async def _runner(event, coro, result, timeout=None):
         event.set()
 
 
-if PY36:
-    grl = asyncio.events._get_running_loop
-else:
-    grl = asyncio.events.get_running_loop
-
-
 def sync(loop, func, *args, timeout=None, **kwargs):
     """
     Make loop run coroutine until it returns. Runs in other thread
@@ -45,7 +39,7 @@ def sync(loop, func, *args, timeout=None, **kwargs):
     if loop is None or loop.is_closed():
         raise RuntimeError("Loop is not running")
     try:
-        loop0 = grl()
+        loop0 = asyncio.events.get_running_loop()
         if loop0 is loop:
             raise NotImplementedError("Calling sync() from within a running loop")
     except RuntimeError:
@@ -240,9 +234,7 @@ async def _run_coros_in_chunks(
         ]
         if callback is not _DEFAULT_CALLBACK:
             [
-                t.add_done_callback(
-                    lambda *_, **__: callback.call("relative_update", 1)
-                )
+                t.add_done_callback(lambda *_, **__: callback.relative_update(1))
                 for t in chunk
             ]
         results.extend(
@@ -320,6 +312,9 @@ class AsyncFileSystem(AbstractFileSystem):
             nofiles=True,
         )
 
+    async def _cp_file(self, path1, path2, **kwargs):
+        raise NotImplementedError
+
     async def _copy(
         self,
         path1,
@@ -418,7 +413,7 @@ class AsyncFileSystem(AbstractFileSystem):
     ):
         # TODO: on_error
         if max_gap is not None:
-            # to be implemented in utils
+            # use utils.merge_offset_ranges
             raise NotImplementedError
         if not isinstance(paths, list):
             raise TypeError
@@ -435,6 +430,9 @@ class AsyncFileSystem(AbstractFileSystem):
         batch_size = batch_size or self.batch_size
         return await _run_coros_in_chunks(coros, batch_size=batch_size, nofiles=True)
 
+    async def _put_file(self, lpath, rpath, **kwargs):
+        raise NotImplementedError
+
     async def _put(
         self,
         lpath,
@@ -476,7 +474,7 @@ class AsyncFileSystem(AbstractFileSystem):
         batch_size = batch_size or self.batch_size
 
         coros = []
-        callback.call("set_size", len(file_pairs))
+        callback.set_size(len(file_pairs))
         for lfile, rfile in file_pairs:
             callback.branch(lfile, rfile, kwargs)
             coros.append(self._put_file(lfile, rfile, **kwargs))
@@ -515,7 +513,7 @@ class AsyncFileSystem(AbstractFileSystem):
         batch_size = kwargs.pop("batch_size", self.batch_size)
 
         coros = []
-        callback.lazy_call("set_size", len, lpaths)
+        callback.set_size(len(lpaths))
         for lpath, rpath in zip(lpaths, rpaths):
             callback.branch(rpath, lpath, kwargs)
             coros.append(self._get_file(rpath, lpath, **kwargs))
@@ -554,7 +552,7 @@ class AsyncFileSystem(AbstractFileSystem):
     async def _info(self, path, **kwargs):
         raise NotImplementedError
 
-    async def _ls(self, path, **kwargs):
+    async def _ls(self, path, detail=True, **kwargs):
         raise NotImplementedError
 
     async def _walk(self, path, maxdepth=None, **kwargs):
@@ -784,9 +782,6 @@ def _dump_running_tasks(
 ):
     import traceback
 
-    if PY36:
-        raise NotImplementedError("Do not call this on Py 3.6")
-
     tasks = [t for t in asyncio.tasks.all_tasks(loop[0]) if not t.done()]
     if printout:
         [task.print_stack() for task in tasks]
