@@ -2,6 +2,7 @@ import base64
 import io
 import itertools
 import logging
+import os
 from functools import lru_cache
 
 import fsspec.core
@@ -13,13 +14,32 @@ except ImportError:
 
 from ..asyn import AsyncFileSystem, sync
 from ..callbacks import _DEFAULT_CALLBACK
-from ..core import filesystem, open
-from ..mapping import get_mapper
+from ..core import filesystem, open, split_protocol
 from ..spec import AbstractFileSystem
 
 logger = logging.getLogger("fsspec.reference")
 
 
+def _first(d):
+    return list(d.values())[0]
+
+
+def _prot_in_references(path, references):
+    ref = references.get(path)
+    if isinstance(ref, (list, tuple)):
+        return split_protocol(ref[0])[0] if ref[0] else ref[0]
+
+
+def _protocol_groups(paths, references):
+    if isinstance(paths, str):
+        return {_prot_in_references(paths, references): [paths]}
+    out = {}
+    for path in paths:
+        protocol = _prot_in_references(path, references)
+        out.setdefault(protocol, []).append(path)
+    return out
+
+
 class ReferenceFileSystem(AsyncFileSystem):
     """View byte ranges of some other file as a file system
 
@@ -56,7 +76,6 @@ class ReferenceFileSystem(AsyncFileSystem):
         template_overrides=None,
         simple_templates=True,
         loop=None,
-        ref_type=None,
         **kwargs,
     ):
         """
@@ -84,15 +103,16 @@ class ReferenceFileSystem(AsyncFileSystem):
             order.
         remote_options : dict
             kwargs to go with remote_protocol
-        fs : file system instance
-            Directly provide a file system, if you want to configure it beforehand. This
-            takes precedence over target_protocol/target_options
+        fs : AbstractFileSystem | dict(str, (AbstractFileSystem | dict))
+            Directly provide a file system(s):
+                - a single filesystem instance
+                - a dict of protocol:filesystem, where each value is either a filesystem
+                  instance, or a dict of kwargs that can be used to create in
+                  instance for the given protocol
+            If this is given, remote_options and remote_protocol are ignored.
         template_overrides : dict
             Swap out any templates in the references file with these - useful for
             testing.
-        ref_type : "json" | "parquet" | "zarr"
-            If None, guessed from URL suffix, defaulting to JSON. Ignored if fo
-            is not a string.
         simple_templates: bool
             Whether templates can be processed with simple replace (True) or if
             jinja  is needed (False, much slower). All reference sets produced by
@@ -105,6 +125,7 @@ class ReferenceFileSystem(AsyncFileSystem):
         self.template_overrides = template_overrides
         self.simple_templates = simple_templates
         self.templates = {}
+        self.fss = {}
         if hasattr(fo, "read"):
             text = fo.read()
         elif isinstance(fo, str):
@@ -113,45 +134,35 @@ class ReferenceFileSystem(AsyncFileSystem):
             else:
                 extra = {}
             dic = dict(**(ref_storage_args or target_options or {}), **extra)
-            if ref_type == "zarr" or fo.endswith("zarr"):
-                import pandas as pd
-                import zarr
-
-                self.dataframe = True
-                m = get_mapper(fo, **dic)
-                z = zarr.open_group(m)
-                assert z.attrs["version"] == 1
-                self.templates = z.attrs["templates"]
-                self.gen = z.attrs.get("gen", None)
-                self.df = pd.DataFrame(
-                    {k: z[k][:] for k in ["key", "data", "url", "offset", "size"]}
-                ).set_index("key")
-            elif ref_type == "parquet" or fo.endswith("parquet"):
-                import fastparquet as fp
-
-                self.dataframe = True
-                with open(fo, "rb", **dic) as f:
-                    pf = fp.ParquetFile(f)
-                    assert pf.key_value_metadata["version"] == 1
-                    self.templates = json.loads(pf.key_value_metadata["templates"])
-                    self.gen = json.loads(pf.key_value_metadata.get("gen", "[]"))
-                    self.df = pf.to_pandas(index="key")
-            else:
-                # text JSON
-                with open(fo, "rb", **dic) as f:
-                    logger.info("Read reference from URL %s", fo)
-                    text = f.read()
+            # text JSON
+            with open(fo, "rb", **dic) as f:
+                logger.info("Read reference from URL %s", fo)
+                text = f.read()
         else:
-            # dictionaries; TODO: allow dataframe here?
+            # dictionaries
             text = fo
         if self.dataframe:
             self._process_dataframe()
         else:
             self._process_references(text, template_overrides)
-        if fs is not None:
-            self.fs = fs
+        if isinstance(fs, dict):
+            self.fss = {
+                k: (
+                    fsspec.filesystem(k.split(":", 1)[0], **opts)
+                    if isinstance(opts, dict)
+                    else opts
+                )
+                for k, opts in fs.items()
+            }
             return
+        if fs is not None:
+            # single remote FS
+            remote_protocol = (
+                fs.protocol[0] if isinstance(fs.protocol, tuple) else fs.protocol
+            )
+
         if remote_protocol is None:
+            # get single protocol from any templates
             for ref in self.templates.values():
                 if callable(ref):
                     ref = ref()
@@ -160,6 +171,7 @@ class ReferenceFileSystem(AsyncFileSystem):
                     remote_protocol = protocol
                     break
         if remote_protocol is None:
+            # get single protocol from references
             for ref in self.references.values():
                 if callable(ref):
                     ref = ref()
@@ -171,11 +183,14 @@ class ReferenceFileSystem(AsyncFileSystem):
         if remote_protocol is None:
             remote_protocol = target_protocol
 
-        self.fs = filesystem(remote_protocol, loop=loop, **(remote_options or {}))
+        fs = fs or filesystem(remote_protocol, loop=loop, **(remote_options or {}))
+        self.fss[remote_protocol] = fs
+        self.fss[None] = fs  # default one
 
     @property
     def loop(self):
-        return self.fs.loop if self.fs.async_impl else self._loop
+        inloop = [fs.loop for fs in self.fss.values() if fs.async_impl]
+        return inloop[0] if inloop else self._loop
 
     def _cat_common(self, path):
         path = self._strip_protocol(path)
@@ -214,44 +229,99 @@ class ReferenceFileSystem(AsyncFileSystem):
         part_or_url, start0, end0 = self._cat_common(path)
         if isinstance(part_or_url, bytes):
             return part_or_url[start:end]
-        return (await self.fs._cat_file(part_or_url, start=start0, end=end0))[start:end]
+        protocol, _ = split_protocol(part_or_url)
+        # TODO: start and end should be passed to cat_file, not sliced
+        return (
+            await self.fss[protocol]._cat_file(part_or_url, start=start0, end=end0)
+        )[start:end]
 
     def cat_file(self, path, start=None, end=None, **kwargs):
         part_or_url, start0, end0 = self._cat_common(path)
         if isinstance(part_or_url, bytes):
             return part_or_url[start:end]
-        return self.fs.cat_file(part_or_url, start=start0, end=end0)[start:end]
+        protocol, _ = split_protocol(part_or_url)
+        # TODO: start and end should be passed to cat_file, not sliced
+        return self.fss[protocol].cat_file(part_or_url, start=start0, end=end0)[
+            start:end
+        ]
 
     def pipe_file(self, path, value, **_):
         """Temporarily add binary data or reference as a file"""
         self.references[path] = value
 
     async def _get_file(self, rpath, lpath, **kwargs):
+        if self.isdir(rpath):
+            return os.makedirs(lpath, exist_ok=True)
         data = await self._cat_file(rpath)
         with open(lpath, "wb") as f:
             f.write(data)
 
     def get_file(self, rpath, lpath, callback=_DEFAULT_CALLBACK, **kwargs):
+        if self.isdir(rpath):
+            return os.makedirs(lpath, exist_ok=True)
         data = self.cat_file(rpath, **kwargs)
-        callback.lazy_call("set_size", len, data)
+        callback.set_size(len(data))
         with open(lpath, "wb") as f:
             f.write(data)
-        callback.lazy_call("absolute_update", len, data)
+        callback.absolute_update(len(data))
 
     def get(self, rpath, lpath, recursive=False, **kwargs):
-        if self.fs.async_impl:
-            return sync(self.loop, self._get, rpath, lpath, recursive, **kwargs)
-        return AbstractFileSystem.get(rpath, lpath, recursive=recursive, **kwargs)
+        if isinstance(lpath, list):
+            # because we have to figure out here which lpath goes with which path
+            # after grouping
+            raise NotImplementedError
+        proto_dict = _protocol_groups(rpath, self.references)
+        for proto, paths in proto_dict.items():
+            if self.fss[proto].async_impl:
+                sync(self.loop, self._get, paths, lpath, recursive, **kwargs)
+            else:
+                AbstractFileSystem.get(
+                    self, paths, lpath, recursive=recursive, **kwargs
+                )
 
-    def cat(self, path, recursive=False, **kwargs):
-        if self.fs.async_impl:
-            return sync(self.loop, self._cat, path, recursive, **kwargs)
-        elif isinstance(path, list):
-            if recursive or any("*" in p for p in path):
-                raise NotImplementedError
-            return {p: AbstractFileSystem.cat_file(self, p, **kwargs) for p in path}
-        else:
-            return AbstractFileSystem.cat_file(self, path)
+    def cat(self, path, recursive=False, on_error="raise", **kwargs):
+        proto_dict = _protocol_groups(path, self.references)
+        out = {}
+        for proto, paths in proto_dict.items():
+            if proto is None:
+                # binary/string
+                for p in paths:
+                    try:
+                        out[p] = AbstractFileSystem.cat_file(self, p, **kwargs)
+                    except Exception as e:
+                        if on_error == "raise":
+                            raise
+                        if on_error == "return":
+                            out[p] = e
+
+            elif self.fss[proto].async_impl:
+                # TODO: asyncio.gather on multiple async FSs
+                out.update(
+                    sync(
+                        self.loop,
+                        self._cat,
+                        paths,
+                        recursive,
+                        on_error=on_error,
+                        **kwargs,
+                    )
+                )
+            elif isinstance(paths, list):
+                if recursive or any("*" in p for p in paths):
+                    raise NotImplementedError
+                for p in paths:
+                    try:
+                        out[p] = AbstractFileSystem.cat_file(self, p, **kwargs)
+                    except Exception as e:
+                        if on_error == "raise":
+                            raise
+                        if on_error == "return":
+                            out[p] = e
+            else:
+                out.update(AbstractFileSystem.cat_file(self, paths))
+        if len(out) == 1 and isinstance(path, str) and "*" not in path:
+            return _first(out)
+        return out
 
     def _process_dataframe(self):
         self._process_templates(self.templates)
@@ -274,7 +344,6 @@ class ReferenceFileSystem(AsyncFileSystem):
 
         if self.templates:
             self.df["url"] = self.df["url"].map(_render_jinja)
-        self._dircache_from_items()
 
     def _process_references(self, references, template_overrides=None):
         if isinstance(references, (str, bytes)):
@@ -288,7 +357,6 @@ class ReferenceFileSystem(AsyncFileSystem):
             raise ValueError(f"Unknown reference spec version: {vers}")
         # TODO: we make dircache by iterating over all entries, but for Spec >= 1,
         #  can replace with programmatic. Is it even needed for mapper interface?
-        self._dircache_from_items()
 
     def _process_references0(self, references):
         """Make reference dict for Spec Version 0"""
@@ -315,7 +383,7 @@ class ReferenceFileSystem(AsyncFileSystem):
                 if v.startswith("base64:"):
                     self.references[k] = base64.b64decode(v[7:])
                 self.references[k] = v
-            else:
+            elif self.templates:
                 u = v[0]
                 if "{{" in u:
                     if self.simple_templates:
@@ -327,6 +395,8 @@ class ReferenceFileSystem(AsyncFileSystem):
                     else:
                         u = _render_jinja(u)
                 self.references[k] = [u] if len(v) == 1 else [u, v[1], v[2]]
+            else:
+                self.references[k] = v
         self.references.update(self._process_gen(references.get("gen", [])))
 
     def _process_templates(self, tmp):
@@ -417,6 +487,8 @@ class ReferenceFileSystem(AsyncFileSystem):
 
     def ls(self, path, detail=True, **kwargs):
         path = self._strip_protocol(path)
+        if not self.dircache:
+            self._dircache_from_items()
         out = self._ls_from_cache(path)
         if out is None:
             raise FileNotFoundError
@@ -425,16 +497,18 @@ class ReferenceFileSystem(AsyncFileSystem):
         return [o["name"] for o in out]
 
     def exists(self, path, **kwargs):  # overwrite auto-sync version
-        try:
-            return self._ls_from_cache(path) is not None
-        except FileNotFoundError:
-            return False
+        return self.isdir(path) or self.isfile(path)
 
     def isdir(self, path):  # overwrite auto-sync version
-        return self.exists(path) and self.info(path)["type"] == "directory"
+        if self.dircache:
+            return path in self.dircache
+        else:
+            # this may be faster than building dircache for single calls, but
+            # by looping will be slow for many calls; could cache it?
+            return any(_.startswith(f"{path}/") for _ in self.references)
 
     def isfile(self, path):  # overwrite auto-sync version
-        return self.exists(path) and self.info(path)["type"] == "file"
+        return path in self.references
 
     async def _ls(self, path, detail=True, **kwargs):  # calls fast sync code
         return self.ls(path, detail, **kwargs)
@@ -452,6 +526,10 @@ class ReferenceFileSystem(AsyncFileSystem):
         out0 = [o for o in out if o["name"] == path]
         if not out0:
             return {"name": path, "type": "directory", "size": 0}
+        if out0[0]["size"] is None:
+            # if this is a whole remote file, update size using remote FS
+            prot, _ = split_protocol(self.references[path][0])
+            out0[0]["size"] = self.fss[prot].size(self.references[path][0])
         return out0[0]
 
     async def _info(self, path, **kwargs):  # calls fast sync code
